@@ -91,12 +91,12 @@ class ConvPruner(BasePruningFunc):
         layer.out_channels = layer.out_channels-len(idxs)
         if not layer.transposed:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone()[keep_idxs])
+                layer.weight.data[keep_idxs])
         else:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone()[:, keep_idxs])
+                layer.weight.data[:, keep_idxs])
         if layer.bias is not None:
-            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data[keep_idxs])
         return layer
 
     def prune_in_channels(self, layer: nn.Module, idxs: Sequence[int]) -> nn.Module:
@@ -107,10 +107,10 @@ class ConvPruner(BasePruningFunc):
             keep_idxs = keep_idxs[:len(keep_idxs)//layer.groups]
         if not layer.transposed:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone()[:, keep_idxs])
+                layer.weight.data[:, keep_idxs])
         else:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone()[keep_idxs])
+                layer.weight.data[keep_idxs])
         # no bias pruning because it does not change the output channels
         return layer
 
@@ -130,9 +130,9 @@ class DepthwiseConvPruner(ConvPruner):
         layer.out_channels = layer.out_channels-len(idxs)
         layer.in_channels = layer.in_channels-len(idxs)
         layer.groups = layer.groups-len(idxs)
-        layer.weight = torch.nn.Parameter(layer.weight.data.clone()[keep_idxs])
+        layer.weight = torch.nn.Parameter(layer.weight.data[keep_idxs])
         if layer.bias is not None:
-            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data[keep_idxs])
         return layer
 
     prune_in_channels = prune_out_channels
@@ -147,9 +147,9 @@ class LinearPruner(BasePruningFunc):
         keep_idxs = list(set(range(layer.out_features)) - set(idxs))
         keep_idxs.sort()
         layer.out_features = layer.out_features-len(idxs)
-        layer.weight = torch.nn.Parameter(layer.weight.data.clone()[keep_idxs])
+        layer.weight = torch.nn.Parameter(layer.weight.data[keep_idxs])
         if layer.bias is not None:
-            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data[keep_idxs])
         return layer
 
     def prune_in_channels(self, layer: nn.Module, idxs: Sequence[int]) -> nn.Module:
@@ -157,7 +157,7 @@ class LinearPruner(BasePruningFunc):
         keep_idxs.sort()
         layer.in_features = layer.in_features-len(idxs)
         layer.weight = torch.nn.Parameter(
-            layer.weight.data.clone()[:, keep_idxs])
+            layer.weight.data[:, keep_idxs])
         return layer
 
     def get_out_channels(self, layer):
@@ -174,12 +174,12 @@ class BatchnormPruner(BasePruningFunc):
         keep_idxs = list(set(range(layer.num_features)) - set(idxs))
         keep_idxs.sort()
         layer.num_features = layer.num_features-len(idxs)
-        layer.running_mean = layer.running_mean.data.clone()[keep_idxs]
-        layer.running_var = layer.running_var.data.clone()[keep_idxs]
+        layer.running_mean = layer.running_mean.data[keep_idxs]
+        layer.running_var = layer.running_var.data[keep_idxs]
         if layer.affine:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone()[keep_idxs])
-            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+                layer.weight.data[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data[keep_idxs])
         return layer
 
     prune_in_channels = prune_out_channels
@@ -212,9 +212,9 @@ class LayernormPruner(BasePruningFunc):
         keep_idxs.sort()
         if layer.elementwise_affine:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone().index_select(pruning_dim, keep_idxs))
+                layer.weight.data.index_select(pruning_dim, keep_idxs))
             layer.bias = torch.nn.Parameter(
-                layer.bias.data.clone().index_select(pruning_dim, keep_idxs))
+                layer.bias.data.index_select(pruning_dim, keep_idxs))
         if pruning_dim != -1:
             layer.normalized_shape = layer.normalized_shape[:pruning_dim] + (
                 keep_idxs.size(0), ) + layer.normalized_shape[pruning_dim+1:]
@@ -238,8 +238,8 @@ class GroupNormPruner(BasePruningFunc):
         layer.num_channels = layer.num_channels-len(idxs)
         if layer.affine:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone()[keep_idxs])
-            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+                layer.weight.data[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data[keep_idxs])
         return layer
     
     prune_in_channels = prune_out_channels
@@ -257,8 +257,8 @@ class InstanceNormPruner(BasePruningFunc):
         layer.num_features = layer.num_features-len(idxs)
         if layer.affine:
             layer.weight = torch.nn.Parameter(
-                layer.weight.data.clone()[keep_idxs])
-            layer.bias = torch.nn.Parameter(layer.bias.data.clone()[keep_idxs])
+                layer.weight.data[keep_idxs])
+            layer.bias = torch.nn.Parameter(layer.bias.data[keep_idxs])
         return layer
 
     prune_in_channels = prune_out_channels
@@ -279,7 +279,7 @@ class PReLUPruner(BasePruningFunc):
         keep_idxs = list(set(range(layer.num_parameters)) - set(idxs))
         keep_idxs.sort()
         layer.num_parameters = layer.num_parameters-len(idxs)
-        layer.weight = torch.nn.Parameter(layer.weight.data.clone()[keep_idxs])
+        layer.weight = torch.nn.Parameter(layer.weight.data[keep_idxs])
         return layer
 
     prune_in_channels = prune_out_channels
@@ -304,7 +304,7 @@ class EmbeddingPruner(BasePruningFunc):
         keep_idxs = list(set(range(num_features)) - set(idxs))
         keep_idxs.sort()
         layer.weight = torch.nn.Parameter(
-            layer.weight.data.clone()[:, keep_idxs])
+            layer.weight.data[:, keep_idxs])
         layer.embedding_dim = len(keep_idxs)
         return layer
 
@@ -337,18 +337,18 @@ class LSTMPruner(BasePruningFunc):
         #for l in range(num_layers):
         for pf in postfix:
             setattr(layer, 'weight_hh_l0'+pf, torch.nn.Parameter(
-                getattr(layer, 'weight_hh_l0'+pf).data.clone()[expanded_keep_idxs]))
+                getattr(layer, 'weight_hh_l0'+pf).data[expanded_keep_idxs]))
             if layer.bias:
                 setattr(layer, 'bias_hh_l0'+pf, torch.nn.Parameter(
-                    getattr(layer, 'bias_hh_l0'+pf).data.clone()[expanded_keep_idxs]))
+                    getattr(layer, 'bias_hh_l0'+pf).data[expanded_keep_idxs]))
             setattr(layer, 'weight_hh_l0'+pf, torch.nn.Parameter(
-                getattr(layer, 'weight_hh_l0'+pf).data.clone()[:, keep_idxs]))
+                getattr(layer, 'weight_hh_l0'+pf).data[:, keep_idxs]))
 
             setattr(layer, 'weight_ih_l0'+pf, torch.nn.Parameter(
-                getattr(layer, 'weight_ih_l0'+pf).data.clone()[expanded_keep_idxs]))
+                getattr(layer, 'weight_ih_l0'+pf).data[expanded_keep_idxs]))
             if layer.bias:
                 setattr(layer, 'bias_ih_l0'+pf, torch.nn.Parameter(
-                    getattr(layer, 'bias_ih_l0'+pf).data.clone()[expanded_keep_idxs]))
+                    getattr(layer, 'bias_ih_l0'+pf).data[expanded_keep_idxs]))
         layer.hidden_size = len(keep_idxs)
 
     def prune_in_channels(self, layer: nn.LSTM, idxs: list):
@@ -356,10 +356,10 @@ class LSTMPruner(BasePruningFunc):
         keep_idxs = list(set(range(num_features)) - set(idxs))
         keep_idxs.sort()
         setattr(layer, 'weight_ih_l0', torch.nn.Parameter(
-                    getattr(layer, 'weight_ih_l0').data.clone()[:, keep_idxs]))
+                    getattr(layer, 'weight_ih_l0').data[:, keep_idxs]))
         if layer.bidirectional:
             setattr(layer, 'weight_ih_l0_reverse', torch.nn.Parameter(
-                    getattr(layer, 'weight_ih_l0_reverse').data.clone()[:, keep_idxs]))
+                    getattr(layer, 'weight_ih_l0_reverse').data[:, keep_idxs]))
         layer.input_size = len(keep_idxs)
 
     def get_out_channels(self, layer):
@@ -377,9 +377,9 @@ class ParameterPruner(BasePruningFunc):
     def prune_out_channels(self, tensor, idxs: list) -> nn.Module:
         keep_idxs = list(set(range(tensor.data.shape[self.pruning_dim])) - set(idxs))
         keep_idxs.sort()
-        tensor.data = torch.index_select(
-            tensor.data, self.pruning_dim, torch.LongTensor(keep_idxs).to(tensor.device))
-        return tensor
+        pruned_parameter = nn.Parameter(torch.index_select(
+            tensor.data, self.pruning_dim, torch.LongTensor(keep_idxs).to(tensor.device)))
+        return pruned_parameter
 
     prune_in_channels = prune_out_channels
 
@@ -400,15 +400,18 @@ class MultiheadAttentionPruner(BasePruningFunc):
     def prune_out_channels(self, layer, idxs: list) -> nn.Module:
         keep_idxs = list(set(range(layer.embed_dim)) - set(idxs))
         keep_idxs.sort()
+
+
         if layer.q_proj_weight is not None:
-            layer.q_proj_weight.data = torch.index_select(
-                layer.q_proj_weight.data, 0, torch.LongTensor(keep_idxs))
+            layer.q_proj_weight = nn.Parameter(torch.index_select(
+                layer.q_proj_weight.data, 0, torch.LongTensor(keep_idxs)))
         if layer.k_proj_weight is not None:
-            layer.q_proj_weight.data = torch.index_select(
-                layer.q_proj_weight.data, 0, torch.LongTensor(keep_idxs))
+            layer.q_proj_weight = nn.Parameter(torch.index_select(
+                layer.q_proj_weight.data, 0, torch.LongTensor(keep_idxs)))
         if layer.v_proj_weight is not None:
-            layer.v_proj_weight.data = torch.index_select(
-                layer.v_proj_weight.data, 0, torch.LongTensor(keep_idxs))
+            layer.v_proj_weight = nn.Parameter(torch.index_select(
+                layer.v_proj_weight.data, 0, torch.LongTensor(keep_idxs)))
+
 
         pruning_idxs_repeated = idxs + \
             [i+layer.embed_dim for i in idxs] + \
@@ -417,44 +420,45 @@ class MultiheadAttentionPruner(BasePruningFunc):
             set(range(3*layer.embed_dim)) - set(pruning_idxs_repeated))
         keep_idxs_3x_repeated.sort()
         if layer.in_proj_weight is not None:
-            layer.in_proj_weight.data = torch.index_select(
-                layer.in_proj_weight.data, 0, torch.LongTensor(keep_idxs_3x_repeated))
-            layer.in_proj_weight.data = torch.index_select(
-                layer.in_proj_weight.data, 1, torch.LongTensor(keep_idxs))
-
+            layer.in_proj_weight = nn.Parameter(torch.index_select(
+                layer.in_proj_weight.data, 0, torch.LongTensor(keep_idxs_3x_repeated)))
+            layer.in_proj_weight = nn.Parameter(torch.index_select(
+                layer.in_proj_weight.data, 1, torch.LongTensor(keep_idxs)))
         if layer.in_proj_bias is not None:
-            layer.in_proj_bias.data = torch.index_select(
-                layer.in_proj_bias.data, 0, torch.LongTensor(keep_idxs_3x_repeated))
+            layer.in_proj_bias = nn.Parameter(torch.index_select(
+                layer.in_proj_bias.data, 0, torch.LongTensor(keep_idxs_3x_repeated)))
+
 
         if layer.bias_k is not None:
-            layer.bias_k.data = torch.index_select(
-                layer.bias_k.data, 2, torch.LongTensor(keep_idxs))
+            layer.bias_k = nn.Parameter(torch.index_select(
+                layer.bias_k.data, 2, torch.LongTensor(keep_idxs)))
         if layer.bias_v is not None:
-            layer.bias_v.data = torch.index_select(
-                layer.bias_v.data, 2, torch.LongTensor(keep_idxs))
+            layer.bias_v = nn.Parameter(torch.index_select(
+                layer.bias_v.data, 2, torch.LongTensor(keep_idxs)))
 
         linear = layer.out_proj
         keep_idxs = list(set(range(linear.out_features)) - set(idxs))
         keep_idxs.sort()
         linear.out_features = linear.out_features-len(idxs)
         linear.weight = torch.nn.Parameter(
-            linear.weight.data.clone()[keep_idxs])
+            linear.weight.data[keep_idxs])
         if linear.bias is not None:
             linear.bias = torch.nn.Parameter(
-                linear.bias.data.clone()[keep_idxs])
+                linear.bias.data[keep_idxs])
         keep_idxs = list(set(range(linear.in_features)) - set(idxs))
         keep_idxs.sort()
         linear.in_features = linear.in_features-len(idxs)
         linear.weight = torch.nn.Parameter(
-            linear.weight.data.clone()[:, keep_idxs])
+            linear.weight.data[:, keep_idxs])
+
         layer.embed_dim = layer.embed_dim - len(idxs)
+        layer.head_dim = layer.embed_dim // layer.num_heads
+        layer.kdim = layer.embed_dim
+        layer.vdim = layer.embed_dim
         return layer
 
     prune_in_channels = prune_out_channels
 
-    # def prune_in_channels(self, layer, idxs: list)-> nn.Module:
-    #    return self.prune_out_channels(layer=layer, idxs=idxs)
-
     def get_out_channels(self, layer):
         return layer.embed_dim
 
